"use strict";
/**
 * @packageDocumentation
 * @module SecurityContextProvider
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useSecurityActions = exports.useSecurityState = exports.SecurityActionContext = exports.SecurityContextProvider = void 0;
var react_1 = __importDefault(require("react"));
var reducer_1 = require("./reducer");
/** @ignore */
var StateContext = react_1.default.createContext(null);
/** @ignore */
var DispatchContext = react_1.default.createContext(null);
exports.SecurityActionContext = DispatchContext;
/**
 * A hook to get the security state (authenticated / not authenticated).
 *
 * NOTE: Must be used inside of a [[SecurityContextProvider]].
 * @category Hooks
 */
var useSecurityState = function () {
    var context = react_1.default.useContext(StateContext);
    if (context === null) {
        throw new Error('useSecurityState must be used within an SecurityContextProvider');
    }
    return context;
};
exports.useSecurityState = useSecurityState;
/**
 * A hook to get the security actions (on authenticated / on not authenticated).
 *
 * NOTE: Must be used inside of a [[SecurityContextProvider]].
 * @category Hooks
 */
var useSecurityActions = function () {
    var context = react_1.default.useContext(DispatchContext);
    if (context === null) {
        throw new Error('useSecurityActions must be used within an SecurityContextProvider');
    }
    return context;
};
exports.useSecurityActions = useSecurityActions;
/**
 *  Provides a wrapper for the security state and security actions.
 *
 *  @public
 *  @category Component
 */
var SecurityContextProvider = function (props) {
    var initialContextState = {
        userId: undefined,
        email: undefined,
        rememberMeDetails: {
            email: undefined,
            rememberMe: undefined,
        },
        isAuthenticatedUser: false,
        isLoading: true,
        isSignOut: false,
        isShowingChangePassword: false,
    };
    var _a = react_1.default.useReducer(reducer_1.reducer, initialContextState), state = _a[0], dispatch = _a[1];
    // Function will not change for children (dispatch does not need to be a dependency)
    var onUserNotAuthenticated = react_1.default.useCallback(function (clearRememberMe, overrideRememberMeEmail) {
        if (clearRememberMe === void 0) { clearRememberMe = false; }
        dispatch({
            type: 'userNotAuthenticated',
            payload: { clearRememberMe: clearRememberMe, overrideRememberMeEmail: overrideRememberMeEmail },
        });
    }, []);
    var showChangePassword = react_1.default.useCallback(function () {
        dispatch({
            type: 'showChangePassword',
        });
    }, []);
    var hideChangePassword = react_1.default.useCallback(function () {
        dispatch({
            type: 'hideChangePassword',
        });
    }, []);
    // Function will not change for children (dispatch does not need to be a dependency)
    var onUserAuthenticated = react_1.default.useCallback(function (onUserAuthenticatedArgs) {
        dispatch({
            type: 'userAuthenticated',
            payload: onUserAuthenticatedArgs,
        });
    }, []);
    // Context value will not change unless a sub function is changed
    var memoizedAuthHelper = react_1.default.useMemo(function () {
        var authHelper = {
            onUserAuthenticated: onUserAuthenticated,
            onUserNotAuthenticated: onUserNotAuthenticated,
            showChangePassword: showChangePassword,
            hideChangePassword: hideChangePassword,
        };
        return authHelper;
    }, [onUserAuthenticated, onUserNotAuthenticated, showChangePassword, hideChangePassword]);
    return (react_1.default.createElement(StateContext.Provider, { value: state },
        react_1.default.createElement(DispatchContext.Provider, { value: memoizedAuthHelper }, props.children)));
};
exports.SecurityContextProvider = SecurityContextProvider;
